@@ -129,21 +129,7 @@ ts_unregister_func(void *data)
 static TSRel*
 relcache_lookup(TSRewriteContext *ctx)
 {
-#define ATTRDEFQUERY (Pgversion(ctx->backend)->major >= 73 ? \
-	"SELECT attname, pg_get_expr(d.adbin, d.adrelid), coalesce((pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
-    " , false)" \
-	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
-	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
-	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.relname = '%s'" \
-	" ORDER BY a.attnum" : \
-	"SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
+#define ATTRDEFQUERY "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
 	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
 	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
 	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
@@ -155,23 +141,9 @@ relcache_lookup(TSRewriteContext *ctx)
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
 	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.relname = '%s'" \
-	" ORDER BY a.attnum")
+	" ORDER BY a.attnum"
 
-#define ATTRDEFQUERY2 (Pgversion(ctx->backend)->major >= 73 ? \
-	"SELECT attname, pg_get_expr(d.adbin, d.adrelid), coalesce((pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
-    " , false)" \
-	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
-	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
-	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = pgpool_regclass('%s')" \
-	" ORDER BY a.attnum" : \
-	"SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
+#define ATTRDEFQUERY2 "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
 	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
 	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
 	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
@@ -183,23 +155,9 @@ relcache_lookup(TSRewriteContext *ctx)
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
 	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = pgpool_regclass('%s')" \
-	" ORDER BY a.attnum")
+	" ORDER BY a.attnum"
 
-#define ATTRDEFQUERY3 (Pgversion(ctx->backend)->major >= 73 ? \
-	"SELECT attname, pg_get_expr(d.adbin, d.adrelid), coalesce((pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
-    " , false)" \
-	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
-	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
-	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = to_regclass('%s')" \
-	" ORDER BY a.attnum" : \
-	"SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
+#define ATTRDEFQUERY3 "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
 	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
 	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
 	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
@@ -211,7 +169,7 @@ relcache_lookup(TSRewriteContext *ctx)
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
 	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = to_regclass('%s')" \
-	" ORDER BY a.attnum")
+	" ORDER BY a.attnum"
 
 	char *query;
 	char *table_name;
@@ -368,7 +326,7 @@ rewrite_timestamp_walker(Node *node, void *context)
 					TypeCast	*tc = makeNode(TypeCast);
 					tc->arg = makeTsExpr(ctx);
 					tc->typeName = SystemTypeName("text");
-
+	
 					fcall->funcname = SystemFuncName("timestamptz");
 					fcall->args = list_make1(tc);
 					ctx->rewrite = true;
@@ -758,6 +716,8 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 		stmt = ((PrepareStmt *) node)->query;
 		ctx.rewrite_to_params = true;
 	}
+	else if (IsA(node, CopyStmt) && ((CopyStmt *) node)->query != NULL)
+		stmt = ((CopyStmt *) node)->query;
 	else
 		stmt = node;
 
@@ -820,39 +780,6 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 			}
 		}
 	}
-	else if (IsA(stmt, CreateTableAsStmt))
-	{
-		CreateTableAsStmt *c_stmt = (CreateTableAsStmt *) stmt;
-
-		/*
-		 * CREATE TABLE t1 AS SELECT now();
-		 */
-		if (IsA(c_stmt->query, SelectStmt) && c_stmt->relkind == OBJECT_TABLE)
-		{
-			/* rewrite params */
-			raw_expression_tree_walker(
-									   (Node *) c_stmt->query,
-									   rewrite_timestamp_walker, (void *) &ctx);
-
-			rewrite = ctx.rewrite;
-		}
-	}
-	else if (IsA(stmt, SelectStmt))
-	{
-		SelectStmt *s_stmt = (SelectStmt *) stmt;
-
-		/*
-		 * SELECT now() INTO t1;
-		 */
-		if (s_stmt->intoClause)
-		{
-			raw_expression_tree_walker(
-									   (Node *) s_stmt,
-									   rewrite_timestamp_walker, (void *) &ctx);
-		}
-
-		rewrite = ctx.rewrite;
-	}
 	else
 		;
 
@@ -923,8 +850,7 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 {
 	int16		 tmp2,
 				 num_params,
-				 num_formats,
-				 num_formats_new;
+				 num_formats;
 	int32		 tmp4;
 	int			 i,
 				 ts_len,
@@ -937,11 +863,10 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 
 #ifdef TIMESTAMPDEBUG
 	fprintf(stderr, "message length:%d\n", *len);
-	for(i = 0; i < *len; i++)
+	for(i=0;i<*len;i++)
 	{
 		fprintf(stderr, "%02x ", orig_msg[i]);
 	}
-	fprintf(stderr, "\n");
 #endif
 
 	ts = get_current_timestamp(backend);
@@ -959,78 +884,53 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 	/* allocate extra memory for parameter formats */
 	num_org_params = message->query_context->num_original_params;
 	new_msg = copy_to = (char *) palloc(*len + sizeof(int16) * (message->num_tsparams + num_org_params));
-
 	copy_from = orig_msg;
 
-	/* 1. portal_name */
+	/* portal_name */
 	copy_len = strlen(copy_from) + 1;
 	memcpy(copy_to, copy_from, copy_len);
-	copy_to += copy_len;
-	copy_from += copy_len;
+	copy_to += copy_len; copy_from += copy_len;
 
-	/* 2. stmt_name */
+	/* stmt_name */
 	copy_len = strlen(copy_from) + 1;
 	memcpy(copy_to, copy_from, copy_len);
-	copy_to += copy_len;
-	copy_from += copy_len;
+	copy_to += copy_len; copy_from += copy_len;
 
-	/* 3. format codes */
-
-	/* 3.1. the number of format codes */
+	/* format code */
 	memcpy(&tmp2, copy_from, sizeof(int16));
-	num_formats = ntohs(tmp2);
+	copy_len = sizeof(int16);
+	tmp2 = num_formats = ntohs(tmp2);
 
-	if (num_formats == 0)
+	if (num_formats >= 1)
 	{
-		/*
-		 * If num_formats is 0, the original message has no parameters or the parameter formats are all text,
-		 * so we don't need additional format codes since timestamp parametes use text as its format.
-		 */
-		num_formats_new = 0;
-	}
-	else
-	{
-		/* If num formats is 1, this means the specified format code is applied for all original parameters,
-		 * so enlarge message length to specify format codes for each of original paramters. */
+		/* this means the specified format code is applied all original parameters */
 		if (num_formats == 1)
+		{
 			*len += (num_org_params - 1) * sizeof(int16);
+			tmp2 += num_org_params - 1;
+		}
 
 		/* enlarge message length for timestamp parameter's formats */
 		*len += message->num_tsparams * sizeof(int16);
-		num_formats_new = num_org_params + message->num_tsparams;
+		tmp2 += message->num_tsparams;
 	}
 
-	/* copy the number of format codes */
-	tmp2 = htons(num_formats_new);
-	copy_len = sizeof(int16);
-	memcpy(copy_to, &tmp2, copy_len);
-	copy_to += copy_len;
-	copy_from += copy_len;
+	tmp2 = htons(tmp2);
+	memcpy(copy_to, &tmp2, copy_len);	/* copy number of format codes */
+	copy_to += copy_len; copy_from += copy_len;
+
+	copy_len = num_formats * sizeof(int16);
+
+	memcpy(copy_to, copy_from, copy_len);		/* copy format codes */
+	copy_to += copy_len; copy_from += copy_len;
 
-	/* 3.2. the format codes */
 	if (num_formats >= 1)
 	{
-
-		/* If num_formats is 1, copy the specified format code as numbers of original parameters */
+		/* copy the specified format code as numbers of original parameters */
 		if (num_formats == 1)
 		{
-			int16	org_format_code;
-			memcpy(&org_format_code, copy_from, sizeof(int16));
-			copy_from += copy_len;
-
-			for (i = 0; i < num_org_params; i++)
-			{
-				memcpy(copy_to, &org_format_code, sizeof(int16));
-				copy_to += sizeof(int16);
-			}
-		}
-		/* otherwize, copy the original format codes as they are*/
-		else
-		{
-			copy_len = num_formats * sizeof(int16);
-			memcpy(copy_to, copy_from, copy_len);
-			copy_to += copy_len;
-			copy_from += copy_len;
+			memcpy(copy_to, copy_from, (num_org_params - 1) * sizeof(int16));
+			copy_to += (num_org_params - 1) * sizeof(int16);
 		}
 
 		/* set additional format codes to zero(text) */
@@ -1038,8 +938,6 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 		copy_to += sizeof(int16) * message->num_tsparams;
 	}
 
-	/* 4. parameters */
-
 	/* num params */
 	memcpy(&tmp2, copy_from, sizeof(int16));
 	copy_len = sizeof(int16);
@@ -1048,7 +946,7 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 	memcpy(copy_to, &tmp2, sizeof(int16));
 	copy_to += copy_len; copy_from += copy_len;
 
-	/* original params */
+	/* params */
 	copy_len = 0;
 	for (i = 0; i < num_params; i++)
 	{
@@ -1067,7 +965,6 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 	memcpy(copy_to, copy_from, copy_len);
 	copy_to += copy_len; copy_from += copy_len;
 
-	/* timestamp params */
 	tmp4 = htonl(ts_len);
 	for (i = 0; i < message->num_tsparams; i++)
 	{
@@ -1077,21 +974,12 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 		copy_to += ts_len;
 	}
 
-	/* 5. result format code */
+	/* result format code */
 	memcpy(&tmp2, copy_from, sizeof(int16));
 	copy_len = sizeof(int16);
 	copy_len += sizeof(int16) * ntohs(tmp2);
 	memcpy(copy_to, copy_from, copy_len);
 
-#ifdef TIMESTAMPDEBUG
-	fprintf(stderr, "message length:%d\n", *len);
-	for(i = 0; i < *len; i++)
-	{
-		fprintf(stderr, "%02x ", new_msg[i]);
-	}
-	fprintf(stderr, "\n");
-#endif
-
 	return new_msg;
 }
 
